

; JTAG code by Clemens Helfmeier, CLemensHelfmeier (at) gmx (dot) de
; please report bugs!
; use it as is, NO WARRANTY AT ALL!

; this code communicates with jtag devices
; registers used:
; r24, r25, x, y

.equ	jtag_TLR	= 0		; TEST-LOGIC-RESET
.equ	jtag_RTI	= 1		; RUN-TEST/IDLE
.equ	jtag_SelDR	= 2		; SELECT-DR-SCAN
.equ	jtag_CapDR	= 3		; CAPTURE-DR
.equ	jtag_ShDR	= 4		; SHIFT-DR
.equ	jtag_Ex1DR	= 5		; EXIT1-DR
.equ	jtag_PsDR	= 6		; PAUSE-DR
.equ	jtag_Ex2DR	= 7		; EXIT2-DR
.equ	jtag_UDR	= 8		; UPDATE-DR
.equ	jtag_SelIr	= 9		; SELECT-IR-SCAN
.equ	jtag_CapIR	= 10		; CAPTURE-IR
.equ	jtag_ShIR	= 11		; SHIFT-IR
.equ	jtag_Ex1IR	= 12		; EXIT1-IR
.equ	jtag_PsIR	= 13		; PAUSE-IR
.equ	jtag_Ex2IR	= 14		; EXIT2-IR
.equ	jtag_UIR	= 15		; UPDATE-IR

#define	jtag_rSrcPointer	x
#define jtag_rDestPointer	y
					; the pointer register to use

; register definitions
; all definitions between horizontal lines can be the same registers
; ------
.def	jtag_rStatus		= r24
; ------
.def	jtag_rClockCounter	= r25		; a different register than jtag_rCounter also for counting with ldi support.
; ------
.def	jtag_rCounter		= rArg0		; some register to count with (ldi supported!)
.def	jtag_rReturnCode	= rArg0		; this is the register for the error codes
.def	jtag_rArgument		= rArg0		; this is the register where arguments are passed to the jtag functions
; ------
.def	jtag_rTemp0		= rTemp0	; this is needed by jtag_shift_reg
; ------
.def	jtag_rTemp1		= rTemp1	; this is needed by jtag_shift_reg
; ------

; If the jtag_ReturnErrorCodes is defined, the functions will
; produce error codes. If it is not defined, the functions will just return nothing
;#define jtag_ReturnErrorCodes

; return codes (if enabled, s.a.)
.equ	jtag_RET_OK		= 0x00
.equ	jtag_RET_INVALID_STATE	= 0x01


; io definitinons ...
.equ	jtag_bitTMS	= PC3
.equ	jtag_bitTCK	= PC2
.equ	jtag_bitTDI	= PC5
.equ	jtag_bitTDO	= PC4
; this is a bit tricky: the jtag_bitTDI pin is the "input" into the programmer. Thus connected to the 
; targets chain's last TDO line. Same with jtag_bitTDO, which is connected to the targets chain's first
; TDI line. (This is as if the Programmer would be an elemnt of the chain.

.equ	jtag_portTMS	= PORTC
.equ	jtag_portTCK	= PORTC
.equ	jtag_portTDO	= PORTC
.equ	jtag_portTDI	= PORTC

.equ	jtag_dirTDI	= jtag_portTDI-1
.equ	jtag_pinTDI	= jtag_portTDI-2
.equ	jtag_dirTDO	= jtag_portTDO-1
.equ	jtag_pinTDO	= jtag_portTDO-2
.equ	jtag_dirTMS	= jtag_portTMS-1
.equ	jtag_pinTMS	= jtag_portTMS-2
.equ	jtag_dirTCK	= jtag_portTCK-1
.equ	jtag_pinTCK	= jtag_portTCK-2

#include "jtag_macros.asm"



jtag_Init:
	;initialize the port
	; everything zero
	cbi jtag_portTDI, jtag_bitTDI
	cbi jtag_portTDO, jtag_bitTDO
	sbi jtag_portTMS, jtag_bitTMS		; TMS high puts devices into TEST-LOGIC-RESET
	cbi jtag_portTCK, jtag_bitTCK
	; correct direction
	cbi jtag_dirTDI, jtag_bitTDI
	sbi jtag_dirTDO, jtag_bitTDO
	sbi jtag_dirTMS, jtag_bitTMS
	sbi jtag_dirTCK, jtag_bitTCK
	; put the chain into TEST-LOGIC/RESET state
	rjmp jtag_ResetChain
	


; this software can place the target device into the following states:
;  state				function			possible errors (if enabled)
;	requirements to previous state
;---------------------------------------------------------------------------------------------------
;  CAPTURE-DR, CAPTURE-IR		jtag_toCapDR, jtag_toCapIR	none
;	none
;  SHIFT-DR, SHIFT-IR			jtag_shift_reg			jtag_RET_INVALID_STATE
;	SHIFT-DR, SHIFT-IR, CAPTURE-DR, CAPTURE-IR
;  PAUSE-DR, PAUSE-IR			jtag_toPsR			jtag_RET_INVALID_STATE
;	SHIFT-DR, SHIFT-IR, CAPTURE-DR, CAPTURE-IR,
;	EXIT2-DR, EXIT2-IR
;  EXIT1-DR, EXIT1-IR			jtag_toEx1R			jtag_RET_INVALID_STATE
;	SHIFT-DR, SHIFT-IR, CAPTURE-DR, CAPTURE-IR,
;	EXIT2-DR, EXIT2-IR
;  UPDATE-DR, UPDATE-IR			jtag_toUR			jtag_RET_INVALID_STATE
;	CAPTURE-DR, CAPTURE-IR, SHIFT-DR, SHIFT-IR
;	EXIT1-DR, EXIT1-IR, EXIT2-DR, EXIT2-IR
;	PAUSE-DR, PAUSE-IR
;  RTI					jtag_toRTI			none
;	none
;  TLR					jtag_toTLR			none
;	none
;---------------------------------------------------------------------------------------------------
; thus a complete write/read of 32 bits to/from DR would look like:
; rcall jtag_toCapDR
; ldi rArg0, 32
; ldi xl, low( Source )
; ldi xh, high( Source )
; ldi yl, low( Dest )
; ldi yh, high( Dest )
; rcall jtag_shift_reg
; ; ... test for error here.
; rcall jtag_toUR
; ; ... test for error here.
;------------------------------------------------------------------------------------------
; the transfer could also be done by 4 times 8 bit:
; rcall jtag_toCapDR
; ldi rArg0, 8
; ldi xl, low( Source0 )
; ldi xh, high( Source0 )
; ldi yl, low( Dest0 )
; ldi yh, high( Dest0 )
; rcall jtag_shift_reg		; read 8 bit
; ; ... test for error here.
; ldi rArg0, 8
; ldi xl, low( Source1 )
; ldi xh, high( Source1 )
; ldi yl, low( Dest1 )
; ldi yh, high( Dest1 )
; rcall jtag_shift_reg		; read 8 bit
; ; ... test for error here.
; ldi rArg0, 8
; ldi xl, low( Source2 )
; ldi xh, high( Source2 )
; ldi yl, low( Dest2 )
; ldi yh, high( Dest2 )
; rcall jtag_shift_reg		; read 8 bit
; ; ... test for error here.
; ldi rArg0, 8
; ldi xl, low( Source3 )
; ldi xh, high( Source3 )
; ldi yl, low( Dest3 )
; ldi yh, high( Dest3 )
; rcall jtag_shift_reg		; read 8 bit
; ; ... test for error here.
; rcall jtag_toUR
; ; ... test for error here.
;------------------------------------------------------------------------------------------

	
; --- jtag_toTLR ---
; puts the target in TEST-LOGIC/RESET state
; needs rCounter
jtag_ResetChain:
jtag_toTLR:
	; at least five TCK clocks with TMS high
	jtag_setTMS
	ldi jtag_rCounter, 5
jtag_ResetChain_a:
		jtag_clock
		dec jtag_rCounter
		brne jtag_ResetChain_a
	; now every chip in the chain is in TEST-LOGIC-RESET
	; and the IR is either BYPASS or IDCODE
	ldi jtag_rStatus, jtag_TLR
	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK		; this is always successful :)
	#endif
	ret

; --- jtag_toSelIR ---
; puts the target in SELECT-IR state
; if not in UDR, UIR, RTI, TLR or SelDR state, this function
; needs rCounter (calls jtag_toSelDR -> jtag_toRTI -> jtag_toTLR)
jtag_toSelIR:
	cpi jtag_rStatus, jtag_SelIR
	breq jtag_toSelIR_exit		; if in SelIR -> ok
	
	;SelIR can only be gone to via SelDR
	rcall jtag_toSelDR			; this function can't produce an error
	
	jtag_setTMS
	jtag_clock
	
jtag_toSelIR_exit:
	ldi jtag_rStatus, jtag_SelIR
	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK		; this is always successful :)
	#endif
	ret

; --- jtag_toSelDR ---
; puts the target in SELECT-DR state
; if not in RTI, TLR, UDR or UIR state, this function
; needs rCounter (calls jtag_toRTI -> jtag_toTLR)
jtag_toSelDR:
	cpi jtag_rStatus, jtag_SelDR
	breq jtag_toSelDR_exit		; if in SelDR -> ok
	
	cpi jtag_rStatus, jtag_RTI
	breq jtag_toSelDR_a
	cpi jtag_rStatus, jtag_UDR
	breq jtag_toSelDR_a
	cpi jtag_rStatus, jtag_UIR
	breq jtag_toSelDR_a
		; any other state
		rcall jtag_toRTI		; this function is always successful -> no error check
jtag_toSelDR_a:
		; now in RTI, UDR or UIR
		jtag_setTMS
		jtag_clock
jtag_toSelDR_exit:
	ldi jtag_rStatus, jtag_SelDR
	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK		; this is always successful :)
	#endif
	ret
	
	
; --- jtag_toRTI ---
; puts the target in RUN-TEST/IDLE state
; if the target is not in TLR, UDR or UIR, this function
; needs rCounter (calls jtag_toTLR)
jtag_toRTI:
	cpi jtag_rStatus, jtag_RTI
	breq jtag_toRTI_a		; if in RTI -> do one clock there
	
	cpi jtag_rStatus, jtag_TLR
	breq jtag_toRTI_a
	cpi jtag_rStatus, jtag_UDR
	breq jtag_toRTI_a
	cpi jtag_rStatus, jtag_UIR
	breq jtag_toRTI_a
		; from anywhere else than TLR, UDR, UIR
		rcall jtag_toTLR		; this function is always successful -> no error check
jtag_toRTI_a:
		; from TLR, RTI, UDR or UIR
		jtag_clearTMS
		jtag_clock
jtag_toRTI_exit:
	ldi jtag_rStatus, jtag_RTI
	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK		; this is always successful :)
	#endif
	ret
	
; --- jtag_toCapIR ---
; this function puts the target in CAPTURE-IR state
; if not in RTI, TLR, SelDR, SelIR, UDR or UIR state, this function
; needs rCounter (calls jtag_toSelDR -> jtag_toRTI -> jtag_toTLR)
jtag_toCapIR:
	cpi jtag_rStatus, jtag_SelIR
	breq jtag_shift_ir_SelIR
	
	rcall jtag_toSelDR		; now in SELECT-DR, this can't produce an error
	
	jtag_setTMS
	jtag_clock			; now in SELECT-IR
jtag_shift_ir_SelIR:

	jtag_clearTMS
	jtag_clock			; now in CAPTURE-IR
	
	ldi jtag_rStatus, jtag_CapIR
					; next clock will put out the first bit of IR
	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK		; this is always successful :)
	#endif
	ret
	
; --- jtag_toCapDR ---
; this function puts the target in CAPTURE-DR state
; if not in RTI, TLR, SelDR, UDR or UIR state, this function
; needs rCounter (calls jtag_toSelDR -> jtag_toRTI -> jtag_toTLR)
jtag_toCapDR:
	;this places the controller in CAPTURE-DR state
	rcall jtag_toSelDR		; now in SELECT-DR
	
	jtag_clearTMS
	jtag_clock			; now in CAPTURE-DR
	
	ldi jtag_rStatus, jtag_CapDR
					; next clock will put out the first bit of DR
	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK		; this is always successful :)
	#endif
	ret

; --- jtag_toEx1R ---
; this places the target in either EXIT1-DR or EXIT1-IR depending
; on the state the target is at before the call
; if not in ShDR, ShIR, CapDR, CapIR, Ex2DRor Ex2IR this function will
; FAIL!
jtag_toEx1R:
	; places the controller in EXIT1-I/DR state
	cpi jtag_rStatus, jtag_ShDR
	breq jtag_toEx1R_b
	cpi jtag_rStatus, jtag_ShIR
	breq jtag_toEx1R_b
	cpi jtag_rStatus, jtag_CapDR
	breq jtag_toEx1R_b
	cpi jtag_rStatus, jtag_CapIR
	breq jtag_toEx1R_b
	
	cpi jtag_rStatus, jtag_Ex2DR
	breq jtag_toEx1R_a
	cpi jtag_rStatus, jtag_Ex2IR
	breq jtag_toEx1R_a
	
		; invalid state
		#ifdef jtag_ReturnErrorCodes
			ldi jtag_rReturnCode, jtag_RET_INVALID_STATE
		#endif
		ret
jtag_toEx1R_a:
	; we are in EXIT2-D/IR
	jtag_clearTMS
	jtag_clock			; now in SHIFT-D/IR
jtag_toEx1R_b:
	jtag_setTMS
	jtag_clock			; now in EXIT1-D/IR
	
	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK	; load return code
	#endif
	cpi jtag_rStatus, 9		; update rStatus
	brsh jtag_toEx1R_Ex1IR
jtag_toEx1R_Ex1DR:
		ldi jtag_rStatus, jtag_Ex1DR
		ret
jtag_toEx1R_Ex1IR:
		ldi jtag_rStatus, jtag_Ex1IR
		ret

; --- jtag_toPsIR ---
; this places the target in PAUSE-IR state
; if the error codes are enabled, this function will fail if the
; target is not in CapIR, ShIR, Ex1IR, Ex2IR or PsIR state
; if no error codes are available, the function will go to PsIR
; even if the way is quite nonsense...
; this function could then use rCounter (call TLR)
jtag_toPsIR:
	; places the controller in PAUSE-IR state
	cpi jtag_rStatus, 10
	brlo jtag_toPsIR_a
	cpi jtag_rStatus, 15
	brsh jtag_toPsIR_a
		; the controller is in some IR state 
		rjmp jtag_toPsR
jtag_toPsIR_a:
		; the controller is in a diffrent state
		#ifdef jtag_ReturnErrorCodes
			ldi jtag_rReturnCode, jtag_RET_INVALID_STATE	; if the error codes are enabled
								; return an error if not in some IR-state
			ret
		#else
			rcall jtag_toCapIR				; if no error codes enabled goto
			rjmp jtag_toPsR					; the state in any way (maybe nonsense!)
		#endif

; --- jtag_toPsDR ---
; this places the target in PAUSE-DR state
; if the error codes are enabled, this function will fail if the
; target is not in CapDR, ShDR, Ex1DR, Ex2DR or PsDR state
; if no error codes are available, the function will go to PsDR
; even if the way is quite nonsense...
; this function could then use rCounter (call TLR)
jtag_toPsDR:
	; places the controller in PAUSE-DR state
	cpi jtag_rStatus, 3
	brlo jtag_toPsDR_a
	cpi jtag_rStatus, 8
	brsh jtag_toPsDR_a
		; the controller is in some DR state 
		rjmp jtag_toPsR
jtag_toPsDR_a:
		; the controller is in a diffrent state
		#ifdef jtag_ReturnErrorCodes
			ldi jtag_rReturnCode, jtag_RET_INVALID_STATE	; if the error codes are enabled
								; return an error if not in some DR-state
			ret
		#else
			rcall jtag_toCapDR				; if no error codes enabled goto
			rjmp jtag_toPsR					; the state in any way (maybe nonsense!)
		#endif

; --- jtag_toPsR ---
; this places the target in the PAUSE-IR or PAUSE-DR state, if it was
; in some IR or DR state respectivley.
; if the target was in RTI, TLR, SelIR, SelDR, UDR, UIR the function will fail
jtag_toPsR:
	; places the controller in PAUSE-I/DR state
	cpi jtag_rStatus, jtag_PsDR
	breq jtag_toPsR_exit
	cpi jtag_rStatus, jtag_PsIR
	breq jtag_toPsR_exit
	
	rcall jtag_toEx1R
	
	#ifdef jtag_ReturnErrorCodes
		cpi jtag_rReturnCode, jtag_RET_OK
		brne jtag_toPsR_exit			; error?
	#endif
	
	jtag_clearTMS
	jtag_clock				; now in PAUSE-I/DR
	
	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK
	#endif
	cpi jtag_rStatus, jtag_Ex1DR		; update the rStatus register
	brne jtag_toPsR_a
		ldi jtag_rStatus, jtag_PsDR
		ret
jtag_toPsR_a:
		ldi jtag_rStatus, jtag_PsIR
jtag_toPsR_exit:
		ret
		
	
; --- jtag_toUR ---
; places the target in the UPDATE-IR or UPDATE-DR state depending on the
; state before the call.
; if target was not in Ex1IR, Ex1DR, Ex2DR, Ex2IR, CapDR, CapIR, ShDR, ShIR
; PsDR or PsIR state, the function will fail.
jtag_toUR:

	jtag_setTMS			; from everywhere the TMS line must be set :)
		
	cpi jtag_rStatus, jtag_Ex1DR
	breq jtag_toUR_oneClock
	cpi jtag_rStatus, jtag_Ex1IR
	breq jtag_toUR_oneClock
	cpi jtag_rStatus, jtag_Ex2DR
	breq jtag_toUR_oneClock
	cpi jtag_rStatus, jtag_Ex2IR
	breq jtag_toUR_oneClock
	
	cpi jtag_rStatus, jtag_CapDR
	breq jtag_toUR_twoClock
	cpi jtag_rStatus, jtag_CapIR
	breq jtag_toUR_twoClock
	cpi jtag_rStatus, jtag_ShDR
	breq jtag_toUR_twoClock
	cpi jtag_rStatus, jtag_ShIR
	breq jtag_toUR_twoClock
	cpi jtag_rStatus, jtag_PsDR
	breq jtag_toUR_twoClock
	cpi jtag_rStatus, jtag_PsDR
	breq jtag_toUR_twoClock
	
		#ifdef jtag_ReturnErrorCodes
			ldi jtag_rReturnCode, jtag_RET_INVALID_STATE	; from every other state, this operation is invalid
		#endif
		ret
	
jtag_toUR_twoClock:
	jtag_clock
jtag_toUR_oneClock:				; only one clock until UPDATE-D/IR
	jtag_clock
	
	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK		;everything ok.
	#endif
	cpi jtag_rStatus, 9			; correct the rStatus register
	brsh jtag_toUR_UIR
jtag_toUR_UDR:
		ldi jtag_rStatus, jtag_UDR
		ret
jtag_toUR_UIR:
		ldi jtag_rStatus, jtag_UIR
		ret
	
; --- jtag_shift_reg ---
; this shifts data in/out of the selected register (IR or DR)
; needs jtag_rTemp0, jtag_rTemp1
; number of bits in jtag_rArgument
; jtag_rSrcPointer is the data which should be send
; jtag_rDestPointer points to the destination of read data (these could be the same)
;
; the controller state _must_ be either CAPTURE-DR, CAPTURE-IR,
; SHIFT-DR or SHIFT-IR otherwise the function will fail.
; after the routine finished, the controller state is SHIFT-DR or SHIFT-IR
jtag_shift_reg:	
	
	cpi jtag_rStatus, jtag_CapDR
	breq jtag_shift_reg_OK
	cpi jtag_rStatus, jtag_CapIR
	breq jtag_shift_reg_OK
	cpi jtag_rStatus, jtag_ShDR
	breq jtag_shift_reg_OK
	cpi jtag_rStatus, jtag_ShIR
	breq jtag_shift_reg_OK
	
		; the controller is not in the correct state -> error
		#ifdef jtag_ReturnErrorCodes
			ldi jtag_rReturnCode, jtag_RET_INVALID_STATE
		#endif
		ret
	
jtag_shift_reg_OK:
	jtag_clearTMS				; stay in Shift-D/IR state
	
	inc jtag_rArgument
jtag_shift_reg_c:	
	ldi jtag_rTemp0, 0x80			; place stop bit in input register
	ld jtag_rTemp1, jtag_rSrcPointer+	; load the value to be shifted into D/IR
jtag_shift_reg_a:
		dec jtag_rArgument
		breq jtag_shift_reg_finished	; if all bits read -> exit
	
		jtag_clock			; perform clock (take new data and shift out old data)
		
		lsr jtag_rTemp1
		jtag_Carry2TDO			; put data on output
		
		jtag_TDI2Carry
		ror jtag_rTemp0			; read one bit and shift it into register
		
	brcc jtag_shift_reg_a			; if not last bit for this byte read, continue
	
	st jtag_rDestPointer+, jtag_rTemp0	; store the read byte
	rjmp jtag_shift_reg_c			; load the stop bit again
	
jtag_shift_reg_finished:
	cpi jtag_rTemp0, 0x80
	breq jtag_shift_reg_b
		st jtag_rDestPointer+, rTemp0	; store if not yet stored	
jtag_shift_reg_b:

	; leave controller in shift state
	; it is easy to send more bits by calling shift_reg again then
	cpi jtag_rStatus, jtag_CapDR
	brne jtag_shift_reg_d
		ldi jtag_rStatus, jtag_ShDR
jtag_shift_reg_d:
	cpi jtag_rStatus, jtag_CapIR
	brne jtag_shift_reg_e
		ldi jtag_rStatus, jtag_ShIR
jtag_shift_reg_e:

	#ifdef jtag_ReturnErrorCodes
		ldi jtag_rReturnCode, jtag_RET_OK
	#endif
	ret
