
; by Clemens Helfmeier, ClemensHelfmeier (at) gmx (dot) de
; please report bugs.
; 31.8.05
;

; this code parses the svf file.
;  it uses the y register for input stream (one line at a time)
;  it uses the x and y register for data output (to jtag functions)
; z register is temporarily used.

; the svf requires quite a bit of ram ;-) so here are the location-definitions...
; the "RegLength" defines the length for registers and so too for the maximum number of
; bytes that can be written with one SDR/SIR instruction.
; Header and Trailer instructions are of the same size, even if they are not always needed
; especially in one target chains. this simplifies code extremely.
; the only instructions that perform jtag operations are SDR, SIR, RUNTEST and STATE functions.
; HIR, HDR, TIR, TDR ENDDR, ENDIR only store the specified values

; define DEBUG in order to get additional output and memory dumps after each line (reduces speed)
;#define DEBUG

; the routine will echo a '\n' after each successfully executed command.
; if the command failed, it will put some error message with '\n' at the end out.

.equ	RET_INVALID_CHAR	= 2


.equ	svf_InputStream		= 0x060
.equ	svf_InputStreamLength	= 0x100


.equ	svf_MiscBase		= 0x160
.equ	svf_MiscLength		= 0x010

.equ	svf_RegLength		= 0x10				; the number of bytes per register
.equ	svf_SetLength		= svf_RegLength * 3 + 1 +1	; the number of bytes per register set
.equ	svf_AllRegLength	= svf_SetLength * 6 + svf_RegLength

.equ	svf_ROut		= 0x170				; this is always the output register
.equ	svf_RegBase		= svf_ROut + svf_RegLength
.equ	svf_TDIOffset		= 0x01
.equ	svf_TDOOffset		= svf_TDIOffset + svf_RegLength
.equ	svf_MASKOffset		= svf_TDOOffset + svf_RegLength
.equ	svf_ActionOffset	= 3*svf_RegLength+1

.equ	svf_DR			= svf_RegBase + svf_SetLength * 0 + svf_RegLength * 0
.equ	svf_DRIn		= svf_RegBase + svf_SetLength * 0 + svf_RegLength * 0 + 1
.equ	svf_DRExp		= svf_RegBase + svf_SetLength * 0 + svf_RegLength * 1 + 1
.equ	svf_DRMask		= svf_RegBase + svf_SetLength * 0 + svf_RegLength * 2 + 1
.equ	svf_DRAction		= svf_RegBase + svf_SetLength * 0 + svf_ActionOffset

.equ	svf_IR			= svf_RegBase + svf_SetLength * 1 + svf_RegLength * 0
.equ	svf_IRIn		= svf_RegBase + svf_SetLength * 1 + svf_RegLength * 0 + 1
.equ	svf_IRExp		= svf_RegBase + svf_SetLength * 1 + svf_RegLength * 1 + 1
.equ	svf_IRMask		= svf_RegBase + svf_SetLength * 1 + svf_RegLength * 2 + 1
.equ	svf_IRAction		= svf_RegBase + svf_SetLength * 1 + svf_ActionOffset

.equ	svf_TIR			= svf_RegBase + svf_SetLength * 2 + svf_RegLength * 0
.equ	svf_TIRIn		= svf_RegBase + svf_SetLength * 2 + svf_RegLength * 0 + 1
.equ	svf_TIRExp		= svf_RegBase + svf_SetLength * 2 + svf_RegLength * 1 + 1
.equ	svf_TIRMask		= svf_RegBase + svf_SetLength * 2 + svf_RegLength * 2 + 1
.equ	svf_TIRAction		= svf_RegBase + svf_SetLength * 2 + svf_ActionOffset

.equ	svf_HIR			= svf_RegBase + svf_SetLength * 3 + svf_RegLength * 0
.equ	svf_HIRIn		= svf_RegBase + svf_SetLength * 3 + svf_RegLength * 0 + 1
.equ	svf_HIRExp		= svf_RegBase + svf_SetLength * 3 + svf_RegLength * 1 + 1
.equ	svf_HIRMask		= svf_RegBase + svf_SetLength * 3 + svf_RegLength * 2 + 1
.equ	svf_HIRAction		= svf_RegBase + svf_SetLength * 3 + svf_ActionOffset

.equ	svf_TDR			= svf_RegBase + svf_SetLength * 4 + svf_RegLength * 0
.equ	svf_TDRIn		= svf_RegBase + svf_SetLength * 4 + svf_RegLength * 0 + 1
.equ	svf_TDRExp		= svf_RegBase + svf_SetLength * 4 + svf_RegLength * 1 + 1
.equ	svf_TDRMask		= svf_RegBase + svf_SetLength * 4 + svf_RegLength * 2 + 1
.equ	svf_TDRAction		= svf_RegBase + svf_SetLength * 4 + svf_ActionOffset

.equ	svf_HDR			= svf_RegBase + svf_SetLength * 5 + svf_RegLength * 0
.equ	svf_HDRIn		= svf_RegBase + svf_SetLength * 5 + svf_RegLength * 0 + 1
.equ	svf_HDRExp		= svf_RegBase + svf_SetLength * 5 + svf_RegLength * 1 + 1
.equ	svf_HDRMask		= svf_RegBase + svf_SetLength * 5 + svf_RegLength * 2 + 1
.equ	svf_HDRAction		= svf_RegBase + svf_SetLength * 5 + svf_ActionOffset


.equ	svf_EndDR		= svf_MiscBase
.equ	svf_EndIR		= svf_MiscBase + 1
.equ	svf_StateIdle		= 0
.equ	svf_StateReset		= 1
.equ	svf_StatePause		= 2

.equ	svf_RunTest		= svf_MiscBase + 2
.equ	svf_RunTestLength	= 2

.equ	svf_ActionTDI		= 1
.equ	svf_ActionTDO		= 2
.equ	svf_ActionMASK		= 3

; this results in 0x180 bytes of sram needed.

.def	svf_rInputLength	= r23	; saves the left number of input bytes
					; one indicates no more characters available
					
					
#include "svf_macros.asm"
#ifdef DEBUG
	#include "svf_dbg.asm"
#endif

.macro svf_Reciever_ShiftInstruction
	svf_ldi16 zl, zh, @0
	rcall svf_Reciever_ShiftInstructionFunction
.endm
.macro svf_DoTransfer
	svf_ldi16 xl, xh, @0		; load the source register value
	rcall svf_DoTransferFunction
.endm
.macro svf_PatternError
	svf_ldi16 zl, zh, @0
	rcall svf_WritePatternMatchFunction
.endm
.macro svf_toEndRegState
	lds rArg0, @0
	rcall svf_toEndRegStateFunction
.endm

svf_Init:
	; clear all values correct
	ldi rTemp0, 0x00
	svf_ldi16 rArg0, rArg1, svf_MiscLength + svf_AllRegLength
	svf_ldi16 zl, zh, svf_MiscBase
svf_Init_a:
	st z+, rTemp0
	subi rArg0, 1
	sbci rArg1, 0
	brne svf_Init_a
	
	ret

; this is the reciever function
; it will never end :)
svf_Reciever:
	svf_WriteLine strRecieverStart
svf_RecieverStart:
	; this is a program for recieving svf data
	#ifdef DEBUG
		svf_MemDump
	#endif
	
	ldi rArg0, '\n'
	svf_WriteByte
	
	svf_ReadLine
	svf_RemoveSpaces

	
; --- test for SDR or SIR instruction (handle JTAG!)
	svf_CompareUntilSpace strSDR
	brne svf_Reciever_21
		svf_Reciever_ShiftInstruction svf_DR
		; do the jtag-communication 
		ldi rArg0, 8				; initialize repeat counter
		
svf_Reciever_21_a:
		push rArg0				; save repeat counter
		rcall jtag_toCapDR			; move to CAPTURE-DR
		
		svf_DoTransfer svf_HDR			; shift in the header of DR
		in rArg0, SREG
		push rArg0
		
		svf_DoTransfer svf_DR			; shift in the IR itself
		pop rArg1
		in rArg0, SREG
		and rArg0, rArg1
		push rArg0
		
		svf_DoTransfer svf_TDR			; shift in the trailer of IR
		pop rArg1
		in rArg0, SREG
		and rArg0, rArg1
		
		pop rArg0				; remove repeat counter
		sbrs rArg0, 1
			rjmp svf_Reciever_21_err
		
		; everything successful
		
		svf_ToEndRegState svf_EndDR		; goto enddr state
		rjmp svf_RecieverStart			; next command
		
svf_Reciever_21_err:
		; the pattern didn't match
		push rArg0
		svf_ToEndRegState svf_EndDR		; goto enddr state
		pop rArg0
		dec rArg0
		brne svf_Reciever_21_a			; as long as not all repeats done, repeat
		
		svf_WriteLine strErrorPatternMismatch	; number of repeats done. this instruction has failed
		svf_PatternError svf_DR
		rjmp svf_RecieverStart			; next command

svf_Reciever_21:
	svf_CompareUntilSpace strSIR
	brne svf_Reciever_22
		svf_Reciever_ShiftInstruction svf_IR
		; do the jtag-communication 
		
		ldi rArg0, 8				; initialize repeat counter
svf_Reciever_22_a:
		push rArg0
		rcall jtag_toCapIR			; move to CAPTURE-IR
		
		svf_DoTransfer svf_HIR			; shift in the header of IR
		in rArg0, SREG
		push rArg0
		
		svf_DoTransfer svf_IR			; shift in the IR itself
		pop rArg1
		in rArg0, SREG
		and rArg0, rArg1
		push rArg0
		
		svf_DoTransfer svf_TIR			; shift in the trailer of IR
		pop rArg1
		in rArg0, SREG
		and rArg0, rArg1
		
		pop rArg0				; remove repeat counter
		sbrs rArg0, 1				; test the or'd z-flag
			rjmp svf_Reciever_22_err
		
		; everything successful
		
		svf_ToEndRegState svf_EndIR		; goto endir state
		rjmp svf_RecieverStart			; next command
		
svf_Reciever_22_err:
		; the pattern didn't match
		push rArg0
		svf_ToEndRegState svf_EndIR		; goto endir state
		pop rArg0
		dec rArg0
		brne svf_Reciever_22_a			; as long as not all repeats done, repeat
		
		svf_WriteLine strErrorPatternMismatch	; number of repeats done. this instruction has failed
		svf_PatternError svf_IR
		rjmp svf_RecieverStart			; next command
svf_Reciever_22:

; --- test for RUNTEST instruction
; the runtest instruction is only implemented to produce as many
; clock impulses as specified after the RUNTEST string.
; any SEC, TCK or SCK will be ignored.
	svf_CompareUntilSpace strRUNTEST
	brne svf_Reciever_31
		; this instruction must be followed by a number
		
		#ifdef DEBUG
			svf_WriteLine strRUNTEST
		#endif
		
		svf_ReadNumber svf_RunTest, svf_RunTestLength
		lds rArg1, svf_RunTest
		lds rArg2, svf_RunTest + 1
svf_Reciever_31_a:
		rcall jtag_toRTI
		subi rArg1, 1
		sbci rArg2, 0
		brne svf_Reciever_31_a			; do the number of clocks
		rjmp svf_RecieverStart
svf_Reciever_31:

; --- test for STATE instruction
	svf_CompareUntilSpace strSTATE
	brne svf_Reciever_32

svf_Reciever_32_a:
		dec svf_rInputLength
		breq svf_Reciever_32_b			; zero -> end of line
		inc svf_rInputLength
		
		svf_RemoveSpaces
		svf_CompareUntilSpace strStateRESET
		brne svf_Reciever_32_a1
			; goto RESET
			#ifdef DEBUG
				svf_WriteLine strStateRESET
			#endif
			rcall jtag_toTLR
			rjmp svf_Reciever_32_a
svf_Reciever_32_a1:
		svf_CompareUntilSpace strStateIDLE
		brne svf_Reciever_32_a2
			; goto IDLE
			#ifdef DEBUG
				svf_WriteLine strStateIDLE
			#endif
			rcall jtag_toRTI
			rjmp svf_Reciever_32_a
svf_Reciever_32_a2:
		svf_CompareUntilSpace strStateDRPAUSE
		brne svf_Reciever_32_a3
			; goto DR-PAUSE
			#ifdef DEBUG
				svf_WriteLine strStateDRPAUSE
			#endif
			rcall jtag_toPsDR
			rjmp svf_Reciever_32_a
svf_Reciever_32_a3:
		svf_CompareUntilSpace strStateIRPAUSE
		brne svf_Reciever_32_a4
			; goto IR-PAUSE
			#ifdef DEBUG
				svf_WriteLine strStateIRPAUSE
			#endif
			rcall jtag_toPsIR
			rjmp svf_Reciever_32_a
svf_Reciever_32_a4:
		; unknown state, ignore it.
		#ifdef DEBUG
			svf_WriteLine svf_dbgUnknownState
		#endif
		svf_GotoNextSpace
		rjmp svf_Reciever_32_a
svf_Reciever_32_b:
		rjmp svf_RecieverStart
svf_Reciever_32:


; --- test for ENDDR or ENDIR instruction (only save)
	ldi rTemp1, 0x00
	svf_CompareUntilSpace strENDDR
	breq svf_Reciever_51_a
	ldi rTemp1, 0x01
	svf_CompareUntilSpace strENDIR
	brne svf_Reciever_51
svf_Reciever_51_a:
		svf_RemoveSpaces
		svf_CompareUntilSpace strStateIdle
		brne svf_Reciever_51_b
			ldi rArg0, svf_StateIdle
			rjmp svf_Reciever_51_z
	svf_Reciever_51_b:
		svf_CompareUntilSpace strStateReset
		brne svf_Reciever_51_c
			ldi rArg0, svf_StateReset
			rjmp svf_Reciever_51_z
	svf_Reciever_51_c:
		ld rArg0, y+
		svf_CompareUntilSpace strStateRPause
		brne svf_Reciever_51_d
			ldi rArg0, svf_StatePause
			rjmp svf_Reciever_51_z
	svf_Reciever_51_d:
		ldi rArg0, svf_StateIdle
	svf_Reciever_51_z:
		cpi rTemp1, 0x00
		breq svf_Reciever_51_ENDDR
	svf_Reciever_51_EndIR:
			sts svf_EndIR, rArg0
			rjmp svf_RecieverStart
	svf_Reciever_51_EndDR:
			sts svf_EndDR, rArg0
			rjmp svf_RecieverStart
svf_Reciever_51:

; --- test if this is a Trailing or Header instruction (only save)
	svf_CompareUntilSpace strTIR
	brne svf_Reciever_61
		svf_Reciever_ShiftInstruction svf_TIR
		rjmp svf_RecieverStart
svf_Reciever_61:
	svf_CompareUntilSpace strHIR
	brne svf_Reciever_62
		svf_Reciever_ShiftInstruction svf_HIR
		rjmp svf_RecieverStart
svf_Reciever_62:
	svf_CompareUntilSpace strHDR
	brne svf_Reciever_63
		svf_Reciever_ShiftInstruction svf_HDR
		rjmp svf_RecieverStart
svf_Reciever_63:
	svf_CompareUntilSpace strTDR
	brne svf_Reciever_64
		svf_Reciever_ShiftInstruction svf_TDR
		rjmp svf_RecieverStart
svf_Reciever_64:
	
#ifdef DEBUG
; the test for unknown commands and comments is only for debug purpose.
	svf_CompareUntilSpace strComment1
	brne svf_Reciever_98
		; Comment recieved
		svf_WriteLine svf_dbgComment
		rjmp svf_RecieverStart
svf_Reciever_98:
	svf_CompareUntilSpace strComment0
	brne svf_Reciever_99
		; Comment recieved
		svf_WriteLine svf_dbgComment
		rjmp svf_RecieverStart
svf_Reciever_99:
	svf_WriteLine svf_dbgUnknownCommand
#endif
	rjmp svf_RecieverStart


svf_WritePatternMatchFunction:
	; requires the Reg-Base in z
	ld rArg2, z+				; load the length
	
	
	svf_ldi16 yl, yh, svf_ROut		; claculate mask, expected and output end (lsb) byte
	mov rTemp1, rArg2
svf_WritePatternMatchFunction_a:
		ld rTemp0, z+
		ld rTemp0, y+
		subi rTemp1, 8
		breq svf_WritePatternMatchFunction_b
		brcc svf_WritePatternMatchFunction_a
svf_WritePatternMatchFunction_b:
	svf_pushY				; save the output lsb
	svf_pushZ				; save "base-end"
	
	adiw zl, svf_RegLength
	mov yl, zl
	mov yh, zh				; expected value
	adiw zl, svf_RegLength			; mask
	mov rTemp1, rArg2
	rcall svf_WritePatternMatchFunctiona
	
	ldi rArg0, ' '
	svf_WriteByte
		
	svf_popZ				; base-end
	svf_popY				; output
	adiw zl, svf_RegLength*2		; mask
	mov rTemp1, rArg2
	rcall svf_WritePatternMatchFunctiona
	
	ret
	
svf_WritePatternMatchFunctiona:
		ld rTemp0, -z			; load mask
		ld rArg0, -y			; load value
		and rArg0, rTemp0
		rcall Bin2BCH8
		push rArg0
		mov rArg0, rArg1
		svf_WriteByte
		pop rArg0
		svf_WriteByte			; write the value
	subi rTemp1, 8
	breq svf_WritePatternMatchFunctionb
	brcc svf_WritePatternMatchFunctiona
svf_WritePatternMatchFunctionb:
	ret

svf_DoTransferFunction:
	svf_ldi16 yl, yh, svf_ROut	; load the dest register value
	svf_pushX
	adiw xl, svf_ActionOffset
	ld rArg2, x				; load the action mask
	
	sbrs rArg2, svf_ActionTDI	; if input bits available
		rjmp svf_DoTransfer_noTDI	; if no input bits available
	
		svf_popX
		svf_pushX
		ld rArg0, x+			; load the number of bits and go to send-value (TDI)
		rcall jtag_shift_reg		; shift into register

svf_DoTransfer_noTDI:
	sbrs rArg2, svf_ActionTDO		; if output bits available
		rjmp svf_DoTransfer_noTDO	; if no output bits available
		
		svf_popX
		svf_pushX
		ld rArg1, x+			; load the length
		adiw xl, svf_RegLength		; goto expected with x
		mov zl, xl
		mov zh, xh
		adiw zl, svf_RegLength		; goto mask with z
		svf_ldi16 yl, yh, svf_ROut	; goto output with y
		
		;compare both
svf_DoTransfer_noTDI_a:
			ld rArg0, y+
			ld rTemp0, z+
			ld rTemp1, x+
			and rArg0, rTemp0
			and rTemp1, rTemp0
			
			cp rArg0, rTemp1
			brne svf_DoTransfer_noTDI_b	; on error goto exit
		
			subi rArg1, 8		; 8 bit handled
			brcc svf_DoTransfer_noTDI_a	; as long as no underflow
			
		;successfull compared
		svf_popX
		sez
		ret
svf_DoTransfer_noTDI_b:
		; error.
		svf_popX
		clz
		ret
svf_DoTransfer_noTDO:
	svf_popX
	sez
	ret
	
svf_toEndRegStateFunction:
	cpi rArg0, svf_StateRESET
	brne svf_toEndRegStateFunction_b
		rcall jtag_toTLR
		ret
svf_toEndRegStateFunction_b:
	cpi rArg0, svf_StatePAUSE
	brne svf_toEndRegStateFunction_c
		rcall jtag_toPsR
		ret
svf_toEndRegStateFunction_c:
	; on default goto Idle
	rcall jtag_toUR
	rcall jtag_toRTI
	ret
	
svf_Reciever_ShiftInstructionFunction:
	; this reads something like length TDI (tdi) TDO (tdo) MASK (mask)
	; the base address must be in z
	; the function name is always followed by the length:
	svf_pushZ						; top stack element is always base pointer

	ld rArg2, z					; load the old length
	ldi rArg0, 1
	rcall svf_ReadNumberFunction			; read the length
	
	svf_popZ
	svf_pushZ						; refresh base Pointer
	ld rArg0, z
	cpi rArg0, 0
	breq svf_Reciever_ShiftInstructionFunction_a
	cp rArg0, rArg2
	brne svf_Reciever_ShiftInstructionFunction_a
		; the length didn't change
		svf_popZ
		svf_pushZ
		ldd rArg2, z+svf_ActionOffset		; load old action flag
		rjmp svf_Reciever_ShiftInstructionFunction_b
svf_Reciever_ShiftInstructionFunction_a:
		; the length changed or is zero
		ldi rArg2, 0x00				; clear action flag
		breq svf_Reciever_ShiftInstructionFunction_exit	; if the length is zero -> exit
		rjmp svf_Reciever_ShiftInstructionFunction_b

	; now begin to parse the optional arguments in any order ;)
svf_Reciever_ShiftInstructionFunction_b:
		svf_RemoveSpaces
		svf_CompareUntilSpace strTDI			; is there a tdi-value?
		brne svf_Reciever_ShiftInstructionFunction_a01
			; there is a tdi-value
			ori rArg2, 1<<svf_ActionTDI
			svf_popZ
			svf_pushZ
			adiw zl, svf_TDIOffset			; goto TDI
			ldi rArg0, svf_RegLength		; max length
			rcall svf_ReadNumberFunction		; read the value
			rjmp svf_Reciever_ShiftInstructionFunction_b
svf_Reciever_ShiftInstructionFunction_a01:
		svf_CompareUntilSpace strTDO			; is there a tdo-value?
		brne svf_Reciever_ShiftInstructionFunction_a02
			; there is a tdi-value
			ori rArg2, 1<<svf_ActionTDO
			svf_popZ
			svf_pushZ
			adiw zl, svf_TDOOffset			; goto TDO
			ldi rArg0, svf_RegLength		; max length
			rcall svf_ReadNumberFunction		; read the value
			rjmp svf_Reciever_ShiftInstructionFunction_b
svf_Reciever_ShiftInstructionFunction_a02:
		svf_CompareUntilSpace strMASK			; is there a mask-value?
		brne svf_Reciever_ShiftInstructionFunction_a03
			; there is a tdi-value
			ori rArg2, 1<<svf_ActionMASK
			svf_popZ
			svf_pushZ
			adiw zl, svf_MASKOffset			; goto MASK
			ldi rArg0, svf_RegLength		; max length
			rcall svf_ReadNumberFunction		; read the value
			rjmp svf_Reciever_ShiftInstructionFunction_b
svf_Reciever_ShiftInstructionFunction_a03:
		ld rArg0, y
		cpi rArg0, ';'					; end of line is indicated by ";"
		breq svf_Reciever_ShiftInstructionFunction_exit
		
		; if end of string?
		dec svf_rInputLength
		breq svf_Reciever_ShiftInstructionFunction_exita
		
		; unknown argument, skip the number
		svf_RemoveSpaces
		svf_GotoNextSpace
		rjmp svf_Reciever_ShiftInstructionFunction_b

svf_Reciever_ShiftInstructionFunction_exita:
	inc svf_rInputLength
svf_Reciever_ShiftInstructionFunction_exit:
	svf_popZ						; remove the pointer from stack
	std z+svf_ActionOffset, rArg2
	ret	
	

	
strTIR:
	.db "TIR", 0
strHIR:
	.db "HIR", 0
strTDR:
	.db "TDR", 0
strHDR:
	.db "HDR", 0
strComment0:
	.db "//", 0
strComment1:
	.db "!", 0
strENDDR:
	.db "ENDDR", 0
strENDIR:
	.db "ENDIR", 0
strRUNTEST:
	.db "RUNTEST", 0
strSIR:
	.db "SIR", 0
strSDR:
	.db "SDR", 0
strSTATE:
	.db "STATE", 0
	
strStateIdle:
	.db "IDLE", 0
strStateReset:
	.db "RESET", 0
strStateIRPAUSE:
	.db "IRPAUSE", 0
strStateDRPAUSE:
	.db "DRPAUSE", 0
strStateRPAUSE:
	.db "RPAUSE", 0

strTDI:
	.db "TDI", 0
strTDO:
	.db "TDO", 0
strMASK:
	.db "MASK", 0
		
strErrorPatternMismatch:
	.db "Err Pattern mismatch: ", 0
strErrorJTAG:
	.db "Err on JTAG: ", 0
strRecieverStart:
	.db "SVF Interpreter for AVR by Clemens Helfmeier\n", 0
strErrorNotice:
	.db "Error. Trying again...\n", 0