

; macros for svf file handling


.macro svf_ldi16
	ldi @0, low(@2)
	ldi @1, high(@2)
.endm
.macro svf_pushZ
	push zl
	push zh
.endm
.macro svf_popZ
	pop zh
	pop zl
.endm
.macro svf_pushY
	push yl
	push yh
.endm
.macro svf_popY
	pop yh
	pop yl
.endm
.macro svf_pushX
	push xl
	push xh
.endm
.macro svf_popX
	pop xh
	pop xl
.endm
.macro svf_addi16
	subi @0, low(-@2)
	sbci @1, high(-@2)
.endm


.macro svf_RemoveSpaces
	; removes spaces from input stream
	; uses rArg0
	rcall svf_RemoveSpacesFunction
.endm
.macro svf_CompareUntilSpace
	; compares the next inputstream data until a space is reached
	; to the arguments flash data
	; if equals then the pointer stays behind the compared word,
	; else it returns to the beginning (for next compare)
	svf_ldi16 zl, zh, @0<<1
	rcall svf_CompareUntilSpaceFunction
.endm
.macro svf_ReadNumber
	; reads a number (hex or dec) from the input stream
	; the destination will be @0
	; the max number of bytes to be written to dest is @1
	svf_ldi16 zl, zh, @0
	ldi rArg0, @1
	rcall svf_ReadNumberFunction
.endm


.macro svf_ReadByte
	; reads one byte from the input
	rcall svf_ReadByteFunction
.endm
.macro svf_ReadLine
	; reads until a \n appears on input
	rcall svf_ReadLineFunction
.endm
.macro svf_WriteLine
	;writes one line of text to the output
	svf_ldi16 zl, zh, @0<<1
	rcall svf_WriteLineFunction
.endm
.macro svf_WriteByte
	; this  is for debuggin purpose
	rcall svf_WriteByteFunction
.endm

.macro svf_GotoNextSpace
	rcall svf_GotoNextSpaceFunction
.endm
.macro svf_GotoNextBracket
	rcall svf_GotoNextBracketFunction
.endm


svf_GotoNextSpaceFunction:
	dec svf_rInputLength
	breq svf_GotoNextSpaceFunction_EOS	; increase inputlength to one agin and exit
	
	ld rArg0, y+
	cpi rArg0, ' '
	brne svf_GotoNextSpaceFunction
		; the last char is a space...
	ld rArg0, -y
svf_GotoNextSpaceFunction_EOS:
	inc svf_rInputLength
	ret
	
	
svf_GotoNextBracketFunction:
	dec svf_rInputLength
	breq svf_GotoNextBracketFunction_EOS	; increase inputlength to one agin and exit
	
	ld rArg0, y+
	cpi rArg0, ')'
	brne svf_GotoNextBracketFunction
		; the last char is a bracket ...
	ld rArg0, -y
svf_GotoNextBracketFunction_EOS:
	inc svf_rInputLength
	ret



svf_ReadNumberFunction:
	; reads from inputstream to z
	; max rArg0 bytes in dest.
	push rArg0
	svf_RemoveSpaces
	ld rTemp0, y
	cpi rTemp0, '('
	breq svf_ReadNumberFunction_Hex
	cpi rTemp0, '0'
	brlo svf_ReadNumberFunction_Err
	cpi rTemp0, '9'+1
	brlo svf_ReadNumberFunction_Dec
svf_ReadNumberFunction_Err:
		; the first char isn't either ( or [0..9]
		pop rArg0
		ldi rArg0, RET_INVALID_CHAR
		ret
svf_ReadNumberFunction_Hex:
		rjmp svf_ReadNumberFunction_HexA
svf_ReadNumberFunction_Dec:
		; read the value in decimals
		; this currently supports only 3 character
		svf_GotoNextSpace			; y now points to the first space behind the last digit
		ldi rTemp0, 1
		clr rTemp1
		clr rTemp2
svf_ReadNumberFunction_Dec_a:
			inc svf_rInputLength
			ld rArg0, -y
			
			cpi rArg0, ' '
			breq svf_ReadNumberFunction_Dec_b	; finished on space
			
			subi rArg0, '0'
			
			mul rArg0, rTemp0		; multiply by power of ten
			add rTemp1, r0			; save
			adc rTemp2, r1
		
			ldi rArg0, 10
			mul rArg0, rTemp0
			mov rTemp0, r0			; next power
			
			cpi rTemp0, 1000
		brne svf_ReadNumberFunction_Dec_a	; as long as not finished
			; 3 character read
svf_ReadNumberFunction_Dec_b:
		dec svf_rInputLength
		ld rArg0, y+				; goto behind the space
		
		pop rArg0				; get max length
		st z+, rTemp1
		dec rArg0
		breq svf_ReadNumberFunction_Dec_c	; over max length now?
			st z+, rTemp2			; no -> save the high value too
svf_ReadNumberFunction_Dec_c:		
		svf_GotoNextSpace			; goto behind number
		ret
		
svf_ReadNumberFunction_HexA:
		; read the value in hex
		svf_GotoNextBracket
		
		pop rTemp0					; get number of available space
		clt
		clr rTemp1
		rjmp svf_ReadNumberFunction_Hex_b
		
svf_ReadNumberFunction_Hex_a:
		set
svf_ReadNumberFunction_Hex_b:
		inc svf_rInputLength
		ld rArg0, -y
		
		cpi rArg0, '('
		breq svf_ReadNumberFunction_Hex_c		; finished with the number
		rcall BCH2Bin4					; convert bch to bin
		
		brtc svf_ReadNumberFunction_Hex_NoSwap
			swap rArg0
svf_ReadNumberFunction_Hex_NoSwap:
		or rTemp1, rArg0				; copy to dest
		brtc svf_ReadNumberFunction_Hex_a		; if high nibble is to be read
		
		st z+, rTemp1					; store both nibbles
		clr rTemp1
		clt						; continue with low nibble again
		dec rTemp0
		breq svf_ReadNumberFunction_Hex_d		; if last possible space written in dest register
		rjmp svf_ReadNumberFunction_Hex_b		; loop

svf_ReadNumberFunction_Hex_c:					; finished
		brtc svf_ReadNumberFunction_Hex_d
			st z+, rTemp1				; there is a low nibble in the register -> store
svf_ReadNumberFunction_Hex_d:
		svf_GotoNextSpace			; y now points to the first space behind the last digit
		ret




;---- don't edit!

svf_RemoveSpacesFunction:
	; removes any spaces from the input stream
	dec svf_rInputLength
	breq svf_RemoveSpaces_EOS	; increase inputlength to one agin and exit
	
	ld rArg0, y+
	cpi rArg0, ' '
	breq svf_RemoveSpacesFunction
		; the last char isn't a space...
	ld rArg0, -y
svf_RemoveSpaces_EOS:
	inc svf_rInputLength
	ret
	
	

svf_CompareUntilSpaceFunction:
	svf_pushY
	push svf_rInputLength				;save for recovery
	ldi rArg1, 0xFF
	
svf_CompareUntilSpaceFunction_a:
		dec svf_rInputLength
		breq svf_CompareUntilSpaceFunction_EOS	;end of stream
	
		ld rArg0, y+				; read stream
		lpm rArg1, z+				; read constant
	
		cpi rArg0, ' '				; on space, exit successfully
		breq svf_CompareUntilSpaceFunction_b
		
		cpi rArg1, 0				; the end of constant is reached
		breq svf_CompareUntilSpaceFunction_c
	
		cp rArg0, rArg1
	breq svf_CompareUntilSpaceFunction_a		; as long as they are the same continue

svf_CompareUntilSpaceFunction_mismatch:			; mismatch
	pop svf_rInputLength
	svf_popY
	clz
	ret

svf_CompareUntilSpaceFunction_c:			; end of costant
;	cpi svf_rInputLength, 1
;	breq svf_CompareUntilSpaceFunction_match	; end of stream is also reached.
;	rjmp svf_CompareUntilSpaceFunction_mismatch	; end of stream is not reached and no ' '
	rjmp svf_CompareUntilSpaceFunction_match	; don't compare the length.
							; this is handled by adding a ' ' to the constant
		
svf_CompareUntilSpaceFunction_EOS:			; end of stream reached
	inc svf_rInputLength
	lpm rArg1, z					; test if constant finished here
	
svf_CompareUntilSpaceFunction_b:
	; the stream has a space
	cpi rArg1, 0
	breq svf_CompareUntilSpaceFunction_match	; the constant must be finished with 0 or space now
	cpi rArg1, ' '
	brne svf_CompareUntilSpaceFunction_mismatch

svf_CompareUntilSpaceFunction_match:			; length and bytes matched
	pop rArg0
	pop rArg0					; remove y
	pop rArg0					; remove rInputLength
	sez
	ret
	
svf_ReadLineFunction:
	svf_ldi16 yl, yh, svf_InputStream	; reset address
	clr svf_rInputLength			; reset length
svf_ReadLineFunction_a:
		svf_ReadByte			; read one input byte
		st y+, rArg0			; store data
	
		inc svf_rInputLength		; increase length
		breq svf_ReadLineFunction_EOS	; the stream space is full
	
		cpi rArg0, 10
	brne svf_ReadLineFunction_a		; as long as not \n recieved
svf_ReadLineFunction_EOL:			; end of line reached
	; length is already one more than bytes recieved
	svf_ldi16 yl, yh, svf_InputStream	; reset the address to start
	ret
svf_ReadLineFunction_EOS:
	; stream is full, read until \n recieved, but do not store
	svf_ReadByte
	cpi rArg0, '\n'
	brne svf_ReadLIneFunction_EOS		; read until new line
	rjmp svf_ReadLineFunction_EOL		; end of line reached


svf_ReadByteFunction:
	sbis UCSRA, RXC
		rjmp svf_ReadByteFunction
	in rArg0, UDR
	ret

svf_WriteLineFunction:
		lpm rArg0, z+
		svf_WriteByte
		cpi rArg0, '\n'
		breq svf_WriteLineFunction_a
		cpi rArg0, 0
	brne svf_WriteLineFunction
svf_WriteLineFunction_a:
	ret

svf_WriteByteFunction:
	sbis UCSRA, UDRE
		rjmp svf_WriteByteFunction
	out UDR, rArg0
	ret